@@ -1,7 +1,7 @@
 /*
  *  templatelistview.cpp  -  widget showing list of alarm templates
  *  Program:  kalarm
- *  Copyright © 2007-2019 David Jarvie <djarvie@kde.org>
+ *  Copyright © 2007-2020 David Jarvie <djarvie@kde.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,9 +36,11 @@ TemplateListView::TemplateListView(QWidget* parent)
     setWhatsThis(i18nc("@info:whatsthis", "The list of alarm templates"));
 }
 
-void TemplateListView::setModel(QAbstractItemModel* model)
+/******************************************************************************
+* Initialize column settings and sizing.
+*/
+void TemplateListView::initSections()
 {
-    EventListView::setModel(model);
     header()->setSectionsMovable(false);
     header()->setStretchLastSection(true);
     header()->setSectionResizeMode(TemplateListModel::TypeColumn, QHeaderView::Fixed);
